uint16_t g_usSendLength = 0u;
uint16_t g_usSendCount = 0u;

uint8_t	g_pucSendBuffer[512];

bool	g_bSendEnd = false;
bool	g_bSendError = false;

/**
 * Function for Send UART
 */
bool	SendUART(const uint8_t * const pucData, 
				 const uint16_t usDataLength)
{
	bool bSuccess = true;

	uint8_t ucIsNested = 0u;
	
	// 1.Enter Critical Region
	app_util_critical_region_enter(&ucIsNested);

	
	g_usSendLength = usDataLength;
	g_usSendCount = 0u;

	
	memcpy(g_pucSendBuffer, pucData, usDataLength);

	g_bSendEnd = false;
	g_bSendError = false;
	
	
	// 2.Put TX FIFO
	PutUART();
	
	
	// 3.Exit Critical Region
	app_util_critical_region_exit(0u);

	return bSuccess;
}

bool IsEndUART(void)
{
	return g_bSendEnd;
}


/**
 * Function for Put UART
 */
void	PutUART(void)
{
	uint32_t ulErrCode = 0u;

	while (1)
	{
		// https://infocenter.nordicsemi.com/index.jsp?topic=%2Fcom.nordic.infocenter.sdk5.v12.2.0%2Fgroup__app__uart.html
		ulErrCode = app_uart_put(g_pucSendBuffer[g_usSendCount]);

		if (NRF_SUCCESS == ulErrCode)
		{
			++g_usSendCount;

			if (g_usSendLength <= g_usSendCount)
			{
				break;
			}
			else
			{
				// DO NOTHING
			}
		}
		else if (NRF_ERROR_NO_MEM == ulErrCode)
		{
			// FIFO Full
			break;
		}
		else
		{
			// UART Error
			g_bSendEnd = true;
			g_bSendError = true;

			break;
		}
	}
}

void uart_event_handle(app_uart_evt_t * p_event)
{
	switch (p_event->evt_type)
	{
		// 4.An event indicating that UART has completed transmission of all available data in the TX FIFO.
	case APP_UART_TX_EMPTY:
		if (g_usSendLength <= g_usSendCount)
		{
			// complete
			g_bSendEnd = true;
		}
		else
		{
			// Put TX FIFO(rest data)
			PutUART();
		}
		break;

	default:
		// DO NOTHING
		break;
	}
}
